//----------------------------------------------------------------------------
// MyTasks State Machine 
//    This breaks the user application into small tasks so that the TCP/IP
//    never has to wait too long.
//----------------------------------------------------------------------------

//------ (global) web page variables used by CustomHTTPApp.c
float tempC, tempF, humidity, dewPoint; // Sensiron values
UINT8 th_setting;         // thermostat setting
char  rbmode[8];          // mode radio button value
char  rbfan[8];           // fan radio button value
UINT8 lblCool;            // Cooling label 0=off 1=on status indicator
UINT8 lblHeat;            // Heating label
UINT8 lblFan;             // Fan label

void MyTasks( void )
{

  static enum {
    smMY_IDLE = 0,    // slow things down a little
    smMY_STEP1,       // get temp & humidity
    smMY_STEP2,       // check for buttons
    smMY_STEP3,       // check for things to turn off or on
    smMY_STEP8,       // LCD updates
    smMY_STEP9,       // configuration updates
  } myAppState = smMY_IDLE;
  
  static DWORD ticks1;  // slow-down timer (See Tick Module in stack doc.)
  static DWORD ticks2;  // temperature reading timer
  static DWORD ticks3;  // backlight timer
  static DWORD ticks4;  // alternate LCD display timer
  
  static UINT8 button1, button2, button3, button4, button5, button6; // button debounce counters
  static char lastMode[8];

  #define LCD_MAIN  0             // default thermostat display
  #define LCD_IP    1             // show netbios name and IP address
  static UINT8 showLCD=LCD_MAIN;  // what do we display on the LCD
  
  char  buffer[17];   			  // work bufffer
  char  lcdBufA1[17],lcdBufA2[17];// lines 0 and 1 for the LCD

  switch( myAppState )
  {
    case smMY_IDLE:               //------ don't work too hard ---------------
      //-- check if we need to turn off backlight
      if( TickGet() - ticks3 >= TICK_SECOND * 5uL ) LEDB_OFF();

      //-- check if we need to turn off alternate LCD display
      if( TickGet() - ticks4 >= TICK_SECOND * 3uL ) showLCD = LCD_MAIN;

      if( TickGet() - ticks1 >= TICK_SECOND/20ul ) 
      {
        ticks1 = TickGet();
        myAppState = smMY_STEP1;
      }
      break;
    
    case smMY_STEP1:          //------ get temp and humidity ----------------- 
      //-- get temp and humdity periodically
      if( TickGet() - ticks2 >= TICK_MINUTE/12ul )  // every 5 seconds
      {
        ticks2 = TickGet();

        sht_rd(&tempC, &humidity);                  // SHT11 library
        debug1("FAN:%s  MODE:%s ",rbfan, rbmode);
        debug1("Sensiron11: %6.2fC  %6.2f%% \r\n", tempC, humidity); dewPoint = 0.0;
      }
      humidity = (int)(humidity);                   // round to integer value
      tempF = (int)(tempC * 9 / 5 + 32);
      myAppState = smMY_STEP2;
      break;
    
    case smMY_STEP2:          //------ check for button press ----------------

      if( !BTNUP_IO && !BTNDN_IO ) // both buttons pressed
      {
        ticks3 = TickGet(); LEDB_ON();      // turn on backlight, start timer
        ticks4 = TickGet(); showLCD=LCD_IP; // show config info
      }		

      if( !BTNUP_IO )       //====== increase temperature
        button1++;
      else
        button1=0;
      if( button1 > 2 )
      {
        ticks3 = TickGet(); LEDB_ON();  // turn on backlight, start timer
        debug2("UP pressed");      
        if( th_setting < 99 ) th_setting++;
        button1=0;
      }

      if( !BTNDN_IO )       //====== decrease temperature
        button2++;
      else
        button2=0;
      if( button2 > 2 )
      {
        ticks3 = TickGet(); LEDB_ON();
        debug2("DOWN pressed");      
        if( th_setting > 32 ) th_setting--;
        button2=0;
      }

      if( !BTNFAN_IO )       //====== toggle fan
        button3++;
      else
        button3=0;
      if( button3 > 2 )
      {
        ticks3 = TickGet(); LEDB_ON(); 
        debug2("FAN pressed");      
        if( strcmp("fanoff",rbfan)==0 )
          strcpy(rbfan,"fanon");
        else
          strcpy(rbfan,"fanoff");
        button3=0;
      }

      if( !BTNHEAT_IO )       //====== turn on heat
        button4++;
      else
        button4=0;
      if( button4 > 2 )
      {
        ticks3 = TickGet(); LEDB_ON(); 
        debug2("HEAT pressed");      
        strcpy(rbmode,"heat");
        button4=0;
      }

      if( !BTNCOOL_IO )       //====== turn on A/C
        button5++;
      else
        button5=0;
      if( button5 > 2 )
      {
        ticks3 = TickGet(); LEDB_ON(); 
        debug2("cool pressed");      
        strcpy(rbmode,"cool");
        button5=0;
      }

      if( !BTNOFF_IO )       //====== turn off
        button6++;
      else
        button6=0;
      if( button6 > 2 )
      {
        ticks3 = TickGet(); LEDB_ON(); 
        debug2("OFF pressed");      
        strcpy(rbmode,"off");
        button6=0;
      }

      myAppState = smMY_STEP3;
      break;
    
    case smMY_STEP3:          //------ check if we need to turn anything on or off
      
      if(strcmp("fanon",rbfan)==0)  { FAN_ON();  lblFan = 1; }
      else                          { FAN_OFF(); lblFan = 0; }

// Rules:
//
//  If rbmode==cool
//    If tempF < th_setting-1 then turn cooling off   (maintains current ooperation it tempF == setting)
//    If tempF > th_setting then turn cooling on
//
//  If rbmode==heat
//    If tempF > th_setting+1 then turn heating off
//    If tempF <= th_setting then turn heating on
//

      if(strcmp("off", rbmode)==0)  { HEAT_OFF(); COOL_OFF(); lblHeat = 0; lblCool = 0; }

      if(strcmp("cool",rbmode)==0)  { lblHeat = 0;
                                      if(tempF < (th_setting)) { HEAT_OFF(); COOL_OFF();  lblCool = 0; }       // in case we go directly from HEAT to COOL, turn off heater
                                      if(tempF > (th_setting)) { HEAT_OFF(); COOL_ON();   lblCool = 1; }  }
      
      if(strcmp("heat",rbmode)==0)  { lblCool = 0;
                                      if(tempF > (th_setting)) { COOL_OFF(); HEAT_OFF();  lblHeat = 0; }
                                      if(tempF < (th_setting)) { COOL_OFF(); HEAT_ON();   lblHeat = 1; }  }
      
      strcpy(lastMode,rbmode);
      myAppState = smMY_STEP8;
      break;
 
    case smMY_STEP8:          //------ update LCD ----------------------------
      
      if(lblFan) 
        sprintf(lcdBufA1,"%3.0f%cF  FAN    %2d",tempF,223,th_setting);
      else  
        sprintf(lcdBufA1,"%3.0f%cF         %2d",tempF,223,th_setting);

      sprintf(lcdBufA2," %2.0f%%         OFF",humidity);
      if(strcmp("cool",rbmode)==0 &&  lblCool) sprintf(lcdBufA2," %2.0f%% COOLING A/C",humidity);
      if(strcmp("cool",rbmode)==0 && !lblCool) sprintf(lcdBufA2," %2.0f%%         A/C",humidity);
      if(strcmp("heat",rbmode)==0 &&  lblHeat) sprintf(lcdBufA2," %2.0f%% HEATING HTR",humidity);
      if(strcmp("heat",rbmode)==0 && !lblHeat) sprintf(lcdBufA2," %2.0f%%         HTR",humidity);

      switch( showLCD ) {
        case LCD_MAIN:
          lcd_xy(0,0);lcd_puts(lcdBufA1);
          lcd_xy(1,0);lcd_puts(lcdBufA2);
          break;
          
        case LCD_IP:
          sprintf(lcdBufA1,"%-16s",&AppConfig.NetBIOSName);
          IPAddressToString( &AppConfig.MyIPAddr, buffer);
          sprintf(lcdBufA2,"%-16s",buffer);
          lcd_xy(0,0);
          lcd_puts(lcdBufA1); //lcd_puts("    ");
          lcd_xy(1,0); 
          lcd_puts(lcdBufA2); //lcd_puts("    ");
          break;
      }  

      myAppState = smMY_STEP9;
      break;
    
    case smMY_STEP9:          //------ check for any configuration changes and save them
      
      myAppState = smMY_IDLE;
      break;
    
    
  } //switch( myAppState ) 
  
}